@@ -2,7 +2,7 @@ package RDF::LinkedData;
 
 use namespace::autoclean;
 
-use RDF::Trine;
+use RDF::Trine qw[iri literal blank statement];
 use RDF::Trine::Serializer;
 use Log::Log4perl qw(:easy);
 use Plack::Response;
@@ -36,37 +36,47 @@ RDF::LinkedData - A simple Linked Data implementation
 
 =head1 VERSION
 
-Version 0.39_2
+Version 0.49_1
 
 =cut
 
-our $VERSION = '0.39_2';
+our $VERSION = '0.49_1';
 
 
 =head1 SYNOPSIS
 
 For just setting this up and get it to run, you would just use the
-L<linked_data.psgi> script in this distribution, which again use
+C<linked_data.psgi> script in this distribution. The usage of that is documented in
 L<Plack::App::RDF::LinkedData>. If you want to try and use this
 directly, you'd do stuff like:
 
 	my $ld = RDF::LinkedData->new(store => $config->{store},
-		  								   endpoint_config => $config->{endpoint},
-											base_uri => $config->{base_uri}
-										  );
+                                 endpoint_config => $config->{endpoint},
+                                 base_uri => $config->{base_uri}
+                                );
 	$ld->namespaces($config->{namespaces}) if ($config->{namespaces});
 	$ld->request($req);
 	return $ld->response($uri)->finalize;
 
 See L<Plack::App::RDF::LinkedData> for a complete example.
 
+=head1 DESCRIPTION
+
+This module is used to create a minimal Linked Data server that can
+serve RDF data out of an L<RDF::Trine::Model>. It will look up URIs in
+the model and do the right thing (known as the 303 dance) and mint
+URLs for that, as well as content negotiation. Thus, you can
+concentrate on URIs for your things, you need not be concerned about
+minting URLs for the pages to serve it.
 
 =head1 METHODS
 
 =over
 
 =item C<< new ( store => $store, model => $model, base_uri => $base_uri, 
-                request => $request, endpoint_config => $endpoint_config ) >>
+                hypermedia => 1, namespaces_as_vocabularies => 1, 
+                request => $request, endpoint_config => $endpoint_config, 
+                void_config => $void_config ) >>
 
 Creates a new handler object based on named parameters, given a store
 config (recommended usage is to pass a hashref of the type that can be
@@ -76,6 +86,13 @@ L<Plack::Request> object (must be passed before you call C<content>)
 and an C<endpoint_config> hashref if you want to have a SPARQL
 Endpoint running using the recommended module L<RDF::Endpoint>.
 
+This module can also provide additional triples to turn the respons
+into a hypermedia type. If you don't want this, set the C<hypermedia>
+argument to false. Currently this entails setting the SPARQL endpoint
+and vocabularies used using the L<VoID vocabulary|http://vocab.deri.ie/void>.
+The latter is very limited at present, all it'll do is use the namespaces
+if you have C<namespaces_as_vocabularies> enabled, which it is by default.
+
 =item C<< BUILD >>
 
 Called by Moose to initialize an object.
@@ -100,6 +117,20 @@ sub BUILD {
  	} else {
 		$self->logger->info('No endpoint config found');
 	}
+
+ 	if ($self->has_void_config) {
+		$self->logger->debug('VoID config found with parameters: ' . Dumper($self->void_config) );
+
+		unless (can_load( modules => { 'RDF::Generator::Void' => 0.02 })) {
+			throw Error -text => "RDF::Generator::Void not installed. Please install or remove its configuration.";
+		}
+		my $dataset_uri = (defined($self->void_config->{dataset_uri}))
+								  ? $self->void_config->{dataset_uri} 
+								  : URI->new($self->base_uri . '#dataset-0')->canonical;
+		$self->void(RDF::Generator::Void->new(inmodel => $self->model, dataset_uri => $dataset_uri));
+ 	} else {
+		$self->logger->info('No VoID config found');
+	}
 }
 
 has store => (is => 'rw', isa => 'HashRef' );
@@ -111,7 +142,7 @@ Returns the RDF::Trine::Model object.
 
 =cut
 
-has model => (is => 'ro', isa => 'RDF::Trine::Model', lazy => 1, builder => '_build_model');
+has model => (is => 'ro', isa => 'RDF::Trine::Model', lazy => 1, builder => '_build_model', handles => ['etag']);
 
 sub _build_model {
 	my $self = shift;
@@ -123,7 +154,7 @@ sub _build_model {
 		}
 		$i++;
 	}
-	my $store	= RDF::Trine::Store->new( $self->store );
+	my $store = RDF::Trine::Store->new( $self->store );
 	return RDF::Trine::Model->new( $store );
 }
 
@@ -134,23 +165,17 @@ Returns or sets the base URI for this handler.
 
 =cut
 
-has base_uri => (is => 'rw', isa => 'Str' );
+has base_uri => (is => 'rw', isa => 'Str', default => '' );
 
+has hypermedia => (is => 'ro', isa => 'Bool', default => 1);
+
+has namespaces_as_vocabularies => (is => 'ro', isa => 'Bool', default => 1);
 
 has endpoint_config => (is => 'rw', traits => [ qw(MooseX::UndefTolerant::Attribute)],
 								isa=>'HashRef', predicate => 'has_endpoint_config');
 
-=item C<< endpoint ( [ $endpoint ] ) >>
-
-Returns the L<RDF::Endpoint> object if it exists or sets it if a
-L<RDF::Endpoint> object is given as parameter. In most cases, it will
-be created for you if you pass a C<endpoint_config> hashref to the
-constructor, so you would most likely not use this method.
-
-=cut
-
-
-has endpoint => (is => 'rw', isa => 'RDF::Endpoint', predicate => 'has_endpoint');
+has void_config => (is => 'rw', traits => [ qw(MooseX::UndefTolerant::Attribute)],
+								isa=>'HashRef', predicate => 'has_void_config');
 
 
 =item C<< request ( [ $request ] ) >>
@@ -162,21 +187,107 @@ Returns the L<Plack::Request> object if it exists or sets it if a L<Plack::Reque
 has request => ( is => 'rw', isa => 'Plack::Request');
 
 
-=item C<< etag ( [ $etag ] ) >>
+=item C<< etag >>
 
 Returns an Etag suitable for use in a HTTP header
 
+=item namespaces ( { skos => 'http://www.w3.org/2004/02/skos/core#', dct => 'http://purl.org/dc/terms/' } )
+
+Gets or sets the namespaces that some serializers use for pretty-printing.
+
 =cut
 
-has etag => ( is => 'ro', isa => 'Str', lazy => 1, builder => '_build_etag');
+has 'namespaces' => (is => 'rw', isa => 'HashRef', default => sub { { rdf => 'http://www.w3.org/1999/02/22-rdf-syntax-ns#' } } );
 
-sub _build_etag {
-	return $_[0]->model->etag;
+
+
+=item C<< response ( $uri ) >>
+
+Will look up what to do with the given URI object and populate the
+response object.
+
+=cut
+
+sub response {
+	my $self = shift;
+	my $uri = URI->new(shift);
+	my $response = Plack::Response->new;
+
+	my $headers_in = $self->request->headers;
+	my $endpoint_path = '/sparql';
+	if ($self->has_endpoint_config && defined($self->endpoint_config->{endpoint_path})) {
+      $endpoint_path = $self->endpoint_config->{endpoint_path};
+	}
+
+	if ($self->has_endpoint && ($uri->path eq $endpoint_path)) {
+      return $self->endpoint->run( $self->request );
+	}
+
+	if ($self->has_void) {
+		my $void_resp = $self->_void_content($uri, $endpoint_path);
+		return $void_resp if (defined($void_resp));
+	}
+
+	my $type = $self->type;
+	$self->type('');
+	my $node = $self->my_node($uri);
+	$self->logger->info("Try rendering '$type' page for subject node: " . $node->as_string);
+	if ($self->count($node) > 0) {
+		if ($type) {
+			my $preds = $self->helper_properties;
+			my $page = $preds->page($node);
+			if (($type eq 'page') && ($page ne $node->uri_value . '/page')) {
+				# Then, we have a foaf:page set that we should redirect to
+				$response->status(301);
+				$response->headers->header('Location' => $page);
+				return $response;
+			}
+
+			$self->logger->debug("Will render '$type' page ");
+			if ($headers_in->can('header') && $headers_in->header('Accept')) {
+				$self->logger->debug('Found Accept header: ' . $headers_in->header('Accept'));
+			} else {
+				$headers_in->header(HTTP::Headers->new('Accept' => 'application/rdf+xml'));
+				if ($headers_in->header('Accept')) {
+					$self->logger->warn('Setting Accept header: ' . $headers_in->header('Accept'));
+				} else {
+					$self->logger->warn('No content type header can be set');
+				}
+			}
+			$response->status(200);
+			my $content = $self->_content($node, $type, $endpoint_path);
+			$response->headers->header('Vary' => join(", ", qw(Accept)));
+			$response->headers->header('ETag' => $self->etag);
+			$response->headers->content_type($content->{content_type});
+			$response->content($content->{body});
+		} else {
+			$response->status(303);
+			my ($ct, $s) = $self->_negotiate($headers_in);
+			return $ct if ($ct->isa('Plack::Response')); # A hack to allow for the failed conneg case
+			my $newurl = $uri . '/data';
+			unless ($s->isa('RDF::Trine::Serializer')) {
+				my $preds = $self->helper_properties;
+				$newurl = $preds->page($node);
+			}
+			$self->logger->debug('Will do a 303 redirect to ' . $newurl);
+			$response->headers->header('Location' => $newurl);
+			$response->headers->header('Vary' => join(", ", qw(Accept)));
+		}
+		return $response;
+	} else {
+		$response->status(404);
+		$response->headers->content_type('text/plain');
+		$response->body('HTTP 404: Unknown resource');
+		return $response;
+	}
+	# We should never get here.
+	$response->status(500);
+	$response->headers->content_type('text/plain');
+	$response->body('HTTP 500: No such functionality.');
+	return $response;
 }
 
 
-
-
 =item C<< helper_properties (  ) >>
 
 Returns the L<RDF::Helper::Properties> object if it exists or sets
@@ -214,7 +325,7 @@ sub my_node {
 	my ($self, $iri) = @_;
     
 	# not happy with this, but it helps for clients that do content sniffing based on filename
-	$iri	=~ s/.(nt|rdf|ttl)$//;
+	$iri =~ s/.(nt|rdf|ttl)$//;
 	$self->logger->info("Subject URI to be used: $iri");
 	return RDF::Trine::Node::Resource->new( $iri );
 }
@@ -232,38 +343,53 @@ sub count {
 	return $self->model->count_statements( $node, undef, undef );
 }
 
-=item C<< content ( $node, $type) >>
+# =item C<< _content ( $node, $type, $endpoint_path) >>
+#
+# Private method to return the a hashref with content for this URI,
+# based on the $node subject, and the type of node, which may be either
+# C<data> or C<page>. In the first case, an RDF document serialized to a
+# format set by content negotiation. In the latter, a simple HTML
+# document will be returned. Finally, you may pass the endpoint path if
+# it is available. The returned hashref has two keys: C<content_type>
+# and C<body>. The former is self-explanatory, the latter contains the
+# actual content.
 
-Will return the a hashref with content for this URI, based on the
-$node subject, and the type of node, which may be either C<data> or
-C<page>. In the first case, an RDF document serialized to a format set
-by content negotiation. In the latter, a simple HTML document will be
-returned. The returned hashref has two keys: C<content_type> and
-C<body>. The former is self-explanatory, the latter contains the
-actual content.
-
-One may argue that a hashref with magic keys should be a class of its
-own, and for that reason, this method should be considered "at
-risk". Currently, it is only used in one place, and it may be turned
-into a private method, get passed the L<Plack::Response> object,
-removed altogether or turned into a role of its own, depending on the
-actual use cases that surfaces in the future.
-
-=cut
-
-
-sub content {
-	my ($self, $node, $type) = @_;
+sub _content {
+	my ($self, $node, $type, $endpoint_path) = @_;
 	my $model = $self->model;
 	my $iter = $model->bounded_description($node);
 	my %output;
 	if ($type eq 'data') {
 		$self->{_type} = 'data';
-		my ($type, $s) = RDF::Trine::Serializer->negotiate('request_headers' => $self->request->headers,
+		my ($ctype, $s) = RDF::Trine::Serializer->negotiate('request_headers' => $self->request->headers,
 																			base => $self->base_uri,
 																			namespaces => $self->namespaces);
-		$output{content_type} = $type;
+		$output{content_type} = $ctype;
+		if ($self->hypermedia) {
+			my $hmmodel = RDF::Trine::Model->temporary_model;
+			if($self->has_endpoint) {
+				$hmmodel->add_statement(statement(iri($node->uri_value . '/data'), 
+															 iri('http://rdfs.org/ns/void#inDataset'), 
+															 blank('void')));
+				$hmmodel->add_statement(statement(blank('void'), 
+															 iri('http://rdfs.org/ns/void#sparqlEndpoint'),
+															 iri($self->base_uri . $endpoint_path)));
+			}
+			if($self->namespaces_as_vocabularies) {
+				$hmmodel->add_statement(statement(iri($node->uri_value . '/data'), 
+															 iri('http://rdfs.org/ns/void#inDataset'), 
+															 blank('void')));
+				foreach my $nsuri (values(%{$self->namespaces})) {
+					$hmmodel->add_statement(statement(blank('void'), 
+																 iri('http://rdfs.org/ns/void#vocabulary'),
+																 iri($nsuri)));
+				}
+			}
+			$iter = $iter->concat($hmmodel->as_stream);
+		}
 		$output{body} = $s->serialize_iterator_to_string ( $iter );
+		$self->logger->trace("Message body is $output{body}");
+
 	} else {
 		$self->{_type} = 'page';
 		my $returnmodel = RDF::Trine::Model->temporary_model;
@@ -271,7 +397,7 @@ sub content {
 			$returnmodel->add_statement($st);
 		}
 		my $preds = $self->helper_properties;
-		my $gen	= RDF::RDFa::Generator->new( style => 'HTML::Pretty',
+		my $gen  = RDF::RDFa::Generator->new( style => 'HTML::Pretty',
 														  title => $preds->title( $node ),
 														  base => $self->base_uri,
 														  namespaces => $self->namespaces);
@@ -284,111 +410,94 @@ sub content {
 
 
 
+=item C<< endpoint ( [ $endpoint ] ) >>
 
-=item C<< response ( $uri ) >>
-
-Will look up what to do with the given URI object and populate the
-response object.
+Returns the L<RDF::Endpoint> object if it exists or sets it if a
+L<RDF::Endpoint> object is given as parameter. In most cases, it will
+be created for you if you pass a C<endpoint_config> hashref to the
+constructor, so you would most likely not use this method.
 
 =cut
 
-sub response {
-	my $self = shift;
-	my $uri = URI->new(shift);
-	my $response = Plack::Response->new;
 
-	my $headers_in = $self->request->headers;
-	my $endpoint_path = '/sparql';
-	if ($self->has_endpoint_config && defined($self->endpoint_config->{endpoint_path})) {
-      my $endpoint_path = $self->endpoint_config->{endpoint_path};
-	}
+has endpoint => (is => 'rw', isa => 'RDF::Endpoint', predicate => 'has_endpoint');
 
-	if ($self->has_endpoint && ($uri->path eq $endpoint_path)) {
-      return $self->endpoint->run( $self->request );
-	}
 
-	my $type = $self->type;
-	$self->type('');
-	my $node = $self->my_node($uri);
-	$self->logger->info("Try rendering '$type' page for subject node: " . $node->as_string);
-	if ($self->count($node) > 0) {
-		if ($type) {
-			my $preds = $self->helper_properties;
-			my $page = $preds->page($node);
-			if (($type eq 'page') && ($page ne $node->uri_value . '/page')) {
-				# Then, we have a foaf:page set that we should redirect to
-				$response->status(301);
-				$response->headers->header('Location' => $page);
-				return $response;
-			}
+=item C<< void ( [ $voidg ] ) >>
 
-			$self->logger->debug("Will render '$type' page ");
-			if ($headers_in->can('header') && $headers_in->header('Accept')) {
-				$self->logger->debug('Found Accept header: ' . $headers_in->header('Accept'));
-			} else {
-				$headers_in->header(HTTP::Headers->new('Accept' => 'application/rdf+xml'));
-				$self->logger->warn('Setting Accept header: ' . $headers_in->header('Accept'));
-			}
-			$response->status(200);
-			my $content = $self->content($node, $type);
-			$response->headers->header('Vary' => join(", ", qw(Accept)));
-			$response->headers->header('ETag' => $self->etag);
-			$response->headers->content_type($content->{content_type});
-			$response->content($content->{body});
-		} else {
-			$response->status(303);
-			my ($ct, $s);
-			eval {
-				($ct, $s) = RDF::Trine::Serializer->negotiate('request_headers' => $headers_in,
-                                                          base => $self->base_uri,
-                                                          namespaces => $self->namespaces,
-																			 extend => {
-																							'text/html'	=> 'html',
-																							'application/xhtml+xml' => 'html'
-																						  }
-																			)
-	      };
-			$self->logger->debug("Got $ct content type");
-			if ($@) {
-				$response->status(406);
-				$response->headers->content_type('text/plain');
-				$response->body('HTTP 406: No serialization available any specified content type');
-				return $response;
-			}
-			my $newurl = $uri . '/data';
-			unless ($s->isa('RDF::Trine::Serializer')) {
-				my $preds = $self->helper_properties;
-				$newurl = $preds->page($node);
-			}
-			$self->logger->debug('Will do a 303 redirect to ' . $newurl);
-			$response->headers->header('Location' => $newurl);
-			$response->headers->header('Vary' => join(", ", qw(Accept)));
-		}
-		return $response;
-	} else {
-		$response->status(404);
+Returns the L<RDF::Generator::Void> object if it exists or sets it if
+a L<RDF::Generator::Void> object is given as parameter. Like
+C<endpoint>, it will be created for you if you pass a C<void_config>
+hashref to the constructor, so you would most likely not use this
+method.
+
+=cut
+
+
+has void => (is => 'rw', isa => 'RDF::Generator::Void', predicate => 'has_void');
+
+
+sub _negotiate {
+	my ($self, $headers_in) = @_;
+	my ($ct, $s);
+	eval {
+		($ct, $s) = RDF::Trine::Serializer->negotiate('request_headers' => $headers_in,
+																	 base_uri => $self->base_uri,
+																	 namespaces => $self->namespaces,
+																	 extend => {
+																					'text/html' => 'html',
+																					'application/xhtml+xml' => 'html'
+																				  }
+																	)
+	};
+	$self->logger->debug("Got $ct content type");
+	if ($@) {
+		my $response = Plack::Response->new;
+		$response->status(406);
 		$response->headers->content_type('text/plain');
-		$response->body('HTTP 404: Unknown resource');
+		$response->body('HTTP 406: No serialization available any specified content type');
 		return $response;
 	}
-	# We should never get here.
-	$response->status(500);
-	$response->headers->content_type('text/plain');
-	$response->body('HTTP 500: No such functionality.');
-	return $response;
+	return ($ct, $s)
 }
 
-
-=item namespaces ( { skos => 'http://www.w3.org/2004/02/skos/core#', dct => 'http://purl.org/dc/terms/' } )
-
-Gets or sets the namespaces that some serializers use for pretty-printing.
-
-=cut
-
-
-
-has 'namespaces' => (is => 'rw', isa => 'HashRef', default => sub { { rdf => 'http://www.w3.org/1999/02/22-rdf-syntax-ns#' } } );
-
+sub _void_content {
+	my ($self, $uri, $endpoint_path) = @_;
+	my $generator = $self->void;
+	my $dataset_uri = URI->new($generator->dataset_uri);
+	my $fragment = $dataset_uri->fragment;
+	$dataset_uri =~ s/(\#$fragment)$//;
+	if ($uri->eq($dataset_uri)) {
+		$generator->urispace($self->base_uri);
+		if ($self->has_endpoint) {
+			$generator->add_endpoints($self->base_uri . $endpoint_path);
+		}
+		my $voidmodel = $generator->generate;
+		my ($ct, $s) = $self->_negotiate($self->request->headers);
+		return $ct if ($ct->isa('Plack::Response')); # A hack to allow for the failed conneg case
+		my $body;
+		if ($s->isa('RDF::Trine::Serializer')) { # Then we just serialize since we have a serializer.
+			$body = $s->serialize_model_to_string($voidmodel);
+		} else {
+			# For (X)HTML, we need to do extra work
+			my $gen = RDF::RDFa::Generator->new( style => 'HTML::Pretty',
+															 title => 'VoID Description',
+															 base => $self->base_uri,
+															 namespaces => $self->namespaces);
+			my $writer = HTML::HTML5::Writer->new( markup => 'xhtml', doctype => DOCTYPE_XHTML_RDFA );
+			$body = encode_utf8( $writer->document($gen->create_document($voidmodel)) );
+		}
+		my $response = Plack::Response->new;
+		$response->status(200);
+		$response->headers->header('Vary' => join(", ", qw(Accept)));
+		$response->headers->header('ETag' => $self->etag);
+		$response->headers->content_type($ct);
+		$response->content($body);
+		return $response;
+	} else {
+		return;
+	}
+}
 
 =back
 
@@ -399,9 +508,7 @@ Kjetil Kjernsmo, C<< <kjetilk@cpan.org> >>
 
 =head1 BUGS
 
-Please report any bugs or feature requests to C<bug-rdf-linkeddata at rt.cpan.org>, or through
-the web interface at L<http://rt.cpan.org/NoAuth/ReportBug.html?Queue=RDF-LinkedData>.  I will be notified, and then you'll
-automatically be notified of progress on your bug as I make changes.
+Please report any bugs using L<github|https://github.com/kjetilk/RDF-LinkedData/issues>
 
 
 =head1 SUPPORT
@@ -424,6 +531,8 @@ L<http://lists.perlrdf.org/listinfo/dev>
 
 =item * Make it read-write hypermedia.
 
+=item * Use a environment variable for config on the command line?
+
 =back
 
 
@@ -443,4 +552,7 @@ under the same terms as Perl itself.
 
 =cut
 
+# TODO : immutable doesn't seem to work with UndefTolerant
+#__PACKAGE__->meta->make_immutable();
+
 1;
